<?php
class Infusionsoft {
    public function __construct(){
        $MyApp = new iSDK;
        $MyApp->cfgCon("purchase");
        $this->app = $MyApp;
        return true;
    }

    public function findByEmail($email){
        $returnFields = array('Id');
        $data = $this->app->findByEmail($email, $returnFields);
        return $data[0]['Id']?$data[0]['Id']:false;
    }

    public function getContactById($id,$returnFields = ['Id']){
        $data = $this->app->loadCon($id,$returnFields);
        if(!is_array($data)){
            return false;
        }
        return $data;
    }

    public function checkByTag($contactId,$tag){
	    $returnFields = ['ContactId'];
	    $query = ['ContactId'=>$contactId, 'GroupId'=>$tag];
	    $contact = $this->app->dsQuery("ContactGroupAssign",10,0,$query,$returnFields);
	    if(!$contact || empty($contact)){
	    	return false;
	    }
	    return true;
    }

    public function applyTagById($contactId,$tags){
    	$tags = explode(',',$tags);
    	foreach($tags as $tagID){
    		$currentTag = intval($tagID);
    		if($tagID){
		        $this->app->grpAssign($contactId, $currentTag);
		    }
	    }
        return true;
    }

    public function removeTagById($contactId,$tags){
    	$tags = explode(',',$tags);
    	foreach($tags as $tagID){
    		$currentTag = intval($tagID);
    		if($tagID){
		        $this->app->grpRemove($contactId, $currentTag);
		    }
	    }
        return true;
    }

    public function getContactsByTag($tagId,$withcustom=false,$returnFields = array('ContactId')){
        if(!$withcustom){
            $tagId = intval($tagId);
            $query = array('GroupId' => $tagId);
            $numberofIterations = $this->app->dsCount('ContactGroupAssign',$query);
            $numberofIterations = ceil($numberofIterations / 1000);
            $contacts = array();
            for($i=0;$i<$numberofIterations;$i++){
                $contacts[] = $this->app->dsQuery("ContactGroupAssign",1000,$i,$query,$returnFields);
            }
            if(empty($contacts) || !is_array($contacts)){
                return array();
            }
            $wholeList = array();
            foreach($contacts as $subspace){
                foreach($subspace as $contact){
                    $wholeList[] = $contact;
                }
            }
            return $wholeList;
        }else{
            $tagId = intval($tagId);
            $query = array('GroupId' => $tagId);
            $fields = array('ContactId');
            $numberofIterations = $this->app->dsCount('ContactGroupAssign',$query);
            $numberofIterations = ceil($numberofIterations / 1000);
            $tags = array();
            for($i=0;$i<$numberofIterations;$i++){
                $tags[] = $this->app->dsQuery("ContactGroupAssign",1000,$i,$query,$fields);
            }
            if(empty($tags) || !is_array($tags)){
                return array();
            }
            $wholeList = array();
            foreach($tags as $subspace){
                foreach($subspace as $contact){
                    $wholeList[] = $contact;
                }
            }

            $ids = [];
            foreach($wholeList as $contactId){
                $ids[]=$contactId['ContactId'];
            }
            $contacts = [];
            $id_chunked = array_chunk($ids,999);
            $chunked_size = count($id_chunked);
            for($i=0;$i<$chunked_size;$i++){
                $contacts[] = $this->app->dsQuery("Contact",1000,0,['Id'=>$id_chunked[$i]],$returnFields);
            }
            $wholeList = array();
            foreach($contacts as $subspace){
                foreach($subspace as $contact){
                    $wholeList[] = $contact;
                }
            }
            return $wholeList;
        }
    }

    public function updateContact($id,$fields = []){
        if(empty($fields) || !is_array($fields)){
            return false;
        }
        return $this->app->updateCon($id,$fields);
    }

    public function createContact($fields=[]){
        if(empty($fields) || !is_array($fields)){
            return false;
        }
        return $this->app->addCon($fields);
    }

    public function createUpdateContact($email,$fields){
    	$contactId = $this->findByEmail($email);
    	if($contactId){
    		return $this->updateContact($contactId,$fields);
	    }else{
    		return $this->createContact($fields);
	    }
    }

    public function addNote($contactId, $addToNotes){
        $returnFields = array('ContactNotes','Id');
        $conDat = $this->app->loadCon($contactId, $returnFields);

        if(!is_array($conDat)){
            return false;
        }
        $notes = $conDat['ContactNotes'] ? $conDat['ContactNotes'] : false;
        if($conDat['Id'] && !$notes){
            $notes = '';
        }
        if(gettype($notes)=='string'){
            $newLine = $notes ? "\n" : "";
            $newNotes = $notes.$newLine.$addToNotes;
            $conDat = array('ContactNotes' => $newNotes);
            $conID = $this->app->updateCon($contactId, $conDat);
            return true;
        }else{
            return false;
        }
    }

    public function getAllTags(){
        $query = array('GroupName' => '%');
        $returnFields = array('Id','GroupName');
        //number of iterations
        $tagCount = $this->app->dsCount("ContactGroup",$query);
        $tagCount = floor($tagCount/1000);
        $i = 0;
        $tags = [];
        for ($i=0; $i<=$tagCount; $i++){
        $tags[] = $this->app->dsQuery("ContactGroup",1000,$i,$query,$returnFields);
        }
        if(empty($tags) || !is_array($tags)){
            return array();
        }
        $wholeList = array();
        foreach($tags as $subspace){
            foreach($subspace as $tag){
                $wholeList[] = $tag;
            }
        }
        return $wholeList;
    }

    /**
     * @param $catId
     * @return array
     */
    public function getLeadsources($catId){
        if(!$catId){
            return false;
        }
        $query = array('LeadSourceCategoryId' => $catId);
        $returnFields = array('Id','Name');
        $leadsources = $this->app->dsQuery("LeadSource",1000,0,$query,$returnFields);
        if(!is_array($leadsources)){
            return false;
        }
        return $leadsources;
    }

    public function updateLeadsource($contactId,$leadSourceId){
        $conDat = array('LeadSourceId' => $leadSourceId);
        $conID = $this->app->updateCon($contactId, $conDat);
        if(intval($conID)){
            return true;
        }else{
            return false;
        }
    }

    public function createTags($tagName,$tagCategory){
	    $data = array('GroupName' => $tagName,
		    'GroupCategoryId'  => intval($tagCategory),
		    'GroupDescription'     => 'Created by event app');
	    $this->app->dsAdd("ContactGroup", $data);
	    return true;
    }

    public function createPaidOrder($title, $contactId, $productId, $productPrice, $howMuchPay = false){
	    $productPrice = (float) $productPrice;
	    if ($howMuchPay === false){
	    	$howMuchPay = $productPrice;
	    }
	    $howMuchPay = (float) $howMuchPay;
	    $howMuchPay = $howMuchPay >= $productPrice ? $productPrice : $howMuchPay;
	    $contactId = (int) $contactId;
	    $productId = (int) $productId;
	    if(!$contactId || !$productId || !$howMuchPay ||!$productPrice){
	    	return false;
	    }

	    //create invoice
	    $currentDate = date("d-m-Y");
	    $pDate = $this->app->infuDate($currentDate);
	    $invoiceId = $this->app->blankOrder($contactId,$title, $pDate, 0, 0);
		if(!intval($invoiceId)){
			return false;
		}

		//add invoice item
	    $result = $this->app->addOrderItem($invoiceId, $productId, 4, $productPrice, 1, "Product Sale", "Event App Product Sale, price=".$productPrice." productId=".$productId);
		if(!$result){
			return false;
		}

		//Mark as paid
	    $result = $this->app->manualPmt($invoiceId,$howMuchPay,$pDate,'Credit Card','CC Swiper',false);
		if(!$result){
			return false;
		}

		return true;
    }
}