<?php
Class model_registrants extends Model{
	public $ajax;
	public $newContactLeadSource = 'AddedDuringEvent';

	public function __construct(){
		parent::__construct();
		$this->ajax = new model_ajax();
		$this->fields = [
			//boot
			'didntAttendTag', 'purchasedTag','attendTag',
			'addNewTag','pendingTag','softwareLow','softwareHigh',
			//live
			'regTagSummit',
			'attendTagSummit',
			'addNewTagSummit',
			'purchasedTagSummit',
			'diamondStartTag','platStartTag','innerCircleStart','goldStartTag','silverStartTag','attendHistorySummit',
			'diamondLifetimeStartTag',
			//both
			'sendText','reviewText'
		];
		return true;
	}

	public function closestSession($sessions){
		if(!$sessions || count($sessions) == 0){
			return 0;
		}
		$today = time()-2*60*60;
		$lowest = 0;
		$lowestId = 0;
		$i = 0;
		foreach($sessions as $session){
			if($i==0){
				$lowest = abs($today - $session->startStamp);
				$lowestId = $session->id;
			}else{
				$value = abs($today - $session->startStamp);
				if($value < $lowest){
					$lowest = $value;
					$lowestId = $session->id;
				}
			}
			$i++;
		}
		return $lowestId;
	}

	public function register($contactId, $session){
		$fields = [
			'_VenueBuilding' => $session->hotelName,
			'_VenueAddress1' => $session->street,
			'_VenuePhone' => $session->hotelPhone,
			'_VenueFullDate' => $session->fullDate,
			'_VenueSessionChosen' => $session->sessionChosen,
			'_SurveyForm' => $session->surveyForm
		];
		//getting tags
		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$bootcampTagId = $event[$session->eventId]->tagId;
		$canceledTag = $event[$session->eventId]->canceledTag ? $event[$session->eventId]->canceledTag:0;
		$tags = $session->tags ? $bootcampTagId.','.$session->tags : $bootcampTagId;

		//apply tags and update contact
		$isoft = new Infusionsoft();
		$isoft->applyTagById($contactId,$tags);
		if($canceledTag){
			$isoft->removeTagById($contactId,$canceledTag);
		}
		return $isoft->updateContact($contactId,$fields);
	}

	public function confirm($contactId, $session){
		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$confirmedTag = $event[$session->eventId]->confirmedTag ? $event[$session->eventId]->confirmedTag:0;
		$canceledTag = $event[$session->eventId]->canceledTag ? $event[$session->eventId]->canceledTag:0;

		if($confirmedTag || $canceledTag){
			$isoft = new Infusionsoft();
		}else{
			return false;
		}
		if($confirmedTag){
			$isoft->applyTagById($contactId,$confirmedTag);
		}
		if($canceledTag){
			$isoft->removeTagById($contactId,$canceledTag);
		}
		return true;
	}

	public function cancel($contactId, $session){
		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$confirmedTag = $event[$session->eventId]->confirmedTag?$event[$session->eventId]->confirmedTag:0;
		$canceledTag = $event[$session->eventId]->canceledTag?$event[$session->eventId]->canceledTag:0;
		$registeredTag = $event[$session->eventId]->tagId?$event[$session->eventId]->tagId:0;

		if($confirmedTag || $canceledTag){
			$isoft = new Infusionsoft();
		}else{
			return false;
		}
		if($confirmedTag){
			$isoft->removeTagById($contactId,$confirmedTag);
		}
		if($canceledTag){
			$isoft->applyTagById($contactId,$canceledTag);
		}
		if($registeredTag && $event[$session->eventId]->type == 'live'){
			$isoft->removeTagById($contactId,$registeredTag);
		}
		return true;
	}

	public function removeRegisteredLive($contactId, $session){
		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$registeredTag = $event[$session->eventId]->tagId?$event[$session->eventId]->tagId:0;

		if($registeredTag && $event[$session->eventId]->type == 'live'){
			$isoft = new Infusionsoft();
			$isoft->removeTagById($contactId,$registeredTag);
		}
		return true;
	}

	public function vip($contactId,$session){

		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$vipTag = $event[$session->eventId]->vipTag?$event[$session->eventId]->vipTag:0;

		if(!$vipTag){
			return false;
		}

		$isoft = new Infusionsoft();
		$isoft->applyTagById($contactId,$vipTag);

		return true;

	}

	public function phoneSale($contactId, $session){
		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$phoneTag = $event[$session->eventId]->phoneTag?$event[$session->eventId]->phoneTag:0;
		$purchasedTag = $event[$session->eventId]->purchasedTag?$event[$session->eventId]->purchasedTag:0;

		if(!$phoneTag){
			return false;
		}

		$isoft = new Infusionsoft();
		$hasPurchased = $isoft->checkByTag($contactId,$purchasedTag);
		if(!$hasPurchased){
			$isoft->applyTagById($contactId,$phoneTag);
			return false;
		}
		return true;
	}

	public function registerLive($contactId, $session){
		$fields = [
			'_LiveEventHotel' => $session->hotelName,
			'_LiveEventAddress' => $session->street,
			'_LiveEventPhone' => $session->hotelPhone,
			'_LiveEventDates' => $session->fullDate,
			'_LiveEventHours' => $session->hours,
		];
		//getting correct date and city
		//'_LiveEventDate' => $session->sessionChosen,
		$date = explode(',',$session->sessionChosen)[0];
		$city = trim(explode(',',$session->street)[1]);

		$state = explode(',',$session->street)[2];
		$parts = explode(' ', $state);
		$last = array_pop($parts);
		$parts = array(implode(' ', $parts), $last);
		$state = trim($parts[0]);

		$fields['_LiveEventCity'] = $city.', '.$state;
		$fields['_LiveEventDate'] = $date;

		//getting tags
		$events = new event();
		$event = $events->model->getEvents($session->eventId);
		$liveTagId = $event[$session->eventId]->tagId;
		$canceledTag = $event[$session->eventId]->canceledTag?$event[$session->eventId]->canceledTag:0;
		$tags = $session->tags ? $liveTagId.','.$session->tags : $liveTagId;

		//applying tags
		$isoft = new Infusionsoft();
		$isoft->applyTagById($contactId,$tags);
		if($canceledTag){
			$isoft->removeTagById($contactId,$canceledTag);
		}
		return $isoft->updateContact($contactId,$fields);
	}

	public function createBootcampContact($markContactId = false,$markTagId){
		$email = $_POST['email'];
		$sessionId = $_POST['session'];
		$eventId = intval($_POST['eventId']);
		$message = $email ? 'New Contact Creation Called (Bootcamp) - '.$email : "Mark Attended Called (Bootcamp) - ContactId:" . $markContactId;
		$this->log->add($message);

		if( (!$email || !$sessionId) && !$markContactId ){
			$this->ajax->setResponse(false,11);
			$this->ajax->getResponse();
			return false;
		}

		if(!$eventId){
			$this->ajax->setResponse(false,11);
			$this->ajax->getResponse();
			return false;
		}

		$event = new event();
		$currentEvent = $event->model->getEvents($eventId);
		$session = new sessions();
		$currentSession = $session->model->getSession($sessionId);

		if(!$currentEvent||!$currentSession){
			$this->ajax->setResponse(false,11);
			$this->ajax->getResponse();
			return false;
		}

		$fields = [
			'Email'     => $email,
			'_VenueBuilding' => $currentSession->hotelName,
			'_VenueAddress1' => $currentSession->street,
			'_VenuePhone' => $currentSession->hotelPhone,
			'_VenueFullDate' => $currentSession->fullDate,
			'_VenueFile' => $currentSession->id,
			'_VenueSessionChosen' => $currentSession->sessionChosen,
		];

		if(isset($_POST['firstname']) && $_POST['firstname']){
			$fields['FirstName'] = $_POST['firstname'];
		}
		if(isset($_POST['lastname']) && $_POST['lastname']){
			$fields['LastName'] = $_POST['lastname'];
		}
		if(isset($_POST['phone']) && $_POST['phone']){
			$fields['Phone1'] = $_POST['phone'];
		}

		$isoft = new Infusionsoft();

		if(!$markContactId){
			$contactId = $isoft->findByEmail($email);
		}else{
			unset($fields['Email']);
			$contactId = $markContactId;
		}
		if(!$contactId){
			$fields['_GaSource'] = $this->newContactLeadSource;
			$contactId = $isoft->createContact($fields);
		}else{
			$contactId = $isoft->updateContact($contactId,$fields);
		}
		$id = $isoft->applyTagById($contactId,$markTagId);
		if(!$id){
			$this->ajax->setResponse(false,10);
		}else{
			$fields['contactId'] = $contactId;
			$fields['eventId'] = $eventId;
			$this->addContactToTableBootcamp($fields);
			$this->ajax->setResponse(true,$contactId);
		}
		$this->ajax->getResponse();
		return true;
	}

	public function addContactToTableBootcamp($data){
		$databaseValues = [
			'ContactId' => intval($data['contactId']),
			'VenueDate' => $data['_VenueFullDate'],
			'eventId' => $data['eventId'],
			'attended' => 'Y'
		];
		if(isset($data['Email'])){
			$databaseValues ['Email'] = $data['Email'];
		}
		if(isset($data['FirstName'])){
			$databaseValues ['FirstName'] = $data['FirstName'];
		}
		if(isset($data['LastName'])){
			$databaseValues ['LastName'] = $data['LastName'];
		}
		if(isset($data['Phone1'])){
			$databaseValues ['Phone'] = $data['Phone1'];
		}
		$isExist = $this->db->hardSelect('SELECT * FROM `registrants` WHERE ContactId='.$databaseValues['ContactId'].' AND eventId='.$databaseValues['eventId']);

		if(empty($isExist)){
			$this->db->insert('registrants',$databaseValues);
		}else{
			$dataNew = ['attended' => 'Y'];
			$this->db->updateRecord('registrants', $dataNew, [ 'ContactId'=>$databaseValues['ContactId'] ] );
		}
		return true;
	}

	public function registrants($id){
		if($_SERVER['REQUEST_METHOD']=='POST'){
			return false;
		}
		$sessions = new sessions();
		$data['sessions'] = $sessions->model->getSessions($id);
		$data['state'] = $this->getStateData();
		$data['registrants'] = $this->getRegistrants($id);
		return $data;
	}

	public function getRegistrants($id){
		$sql = 'SELECT * FROM `registrants` WHERE eventId='.$id;
		return $this->db->hardSelect($sql);
	}

	public function syncRegList($eventId,$ajax=true){
		$this->log->add('Event Registrants Syncing');
		$events = new event();
		$event = $events->model->getEvents($eventId);

		if(!$event){
			$this->ajax->setResponse(false,16);
			$this->ajax->getResponse();
			return false;
		}

		$event = reset($event);

		$tagId = $event->tagId;
		$type = $event->type;
		$attendedTag = $event->attendedTag;
		$purchasedTag = $event->purchasedTag;

		if(!$tagId || !$attendedTag){
			$this->ajax->setResponse(false,17);
			$this->ajax->getResponse();
			return false;
		}
		$returnFields = ['Id','FirstName','LastName','Phone1','Phone2','Email'];
		switch($type){
			case "boot":
				$returnFields[]='_VenueFullDate';
				break;
			case "live":
				break;
			default:
				$returnFields[]='_VenueFullDate';
		}


		$isoft = new Infusionsoft();
		$data = $isoft->getContactsByTag($tagId,true,$returnFields);
		$attendees = $isoft->getContactsByTag($attendedTag,false);
		$buyers = $isoft->getContactsByTag($purchasedTag,false);


		$attendeesIds = [];

		if(!empty($attendees) && is_array($attendees)){
			foreach($attendees as $attendee){
				$attendeesIds[] = $attendee['ContactId'];
			}
		}
		unset($attendees);

		$buyersIds = [];
		if(!empty($buyers) && is_array($buyers)){
			foreach($buyers as $buyer){
				$buyersIds[] = $buyer['ContactId'];
			}
		}
		unset($buyers);
		//add to database
		if(empty($data) && $ajax){
			$this->ajax->setResponse(false,13);
			$this->ajax->getResponse();
			return false;
		}
		if(empty($data)&&!$ajax){
			return false;
		}

		$this->db->hardDelete('DELETE FROM `registrants` WHERE eventId='.$eventId.';');
		$values = '';

		foreach($data as $record){
			$phone = $record['Phone1'] ? $record['Phone1'] : $record['Phone2'];
			$firstName = filter_var($record['FirstName'], FILTER_SANITIZE_STRING);
			$lastName = filter_var($record['LastName'], FILTER_SANITIZE_STRING);
			$date = in_array('_VenueFullDate',$returnFields)?filter_var($record['_VenueFullDate'], FILTER_SANITIZE_STRING):'TBA';
			$attended = in_array($record['Id'],$attendeesIds) ? 'Y':'N';
			$bought = in_array($record['Id'],$buyersIds) ? 'Y':'N';
			$values .= ",('".$record['Id']."','".$firstName."','".$lastName."','".$record['Email']."','".$phone."','".$date."','".$attended."','".$eventId."','".$bought."')";
		}


		$values = ltrim($values,',');
		$this->db->simpleQuery('INSERT INTO `registrants` (ContactId, FirstName, LastName, Email, Phone, VenueDate, attended, eventId, purchased) VALUES '.$values.';');
		if($ajax){
			$this->ajax->setResponse(true,1);
			$this->ajax->getResponse();
		}
		return true;
	}

	public function markAttended() {
		$contactId = intval($_POST['contactId']);
		if (!$contactId) {
			$this->ajax->setResponse(false, 12);
			$this->ajax->getResponse();
			return false;
		}
		$this->log->add("Mark Attended Called (Bootcamp) - ContactId:" . $contactId);
		$data = ['attended' => 'Y'];
		$id = $this->db->updateRecord('registrants', $data, ['ContactId' => $contactId]);
		if ($id) {
			$this->ajax->setResponse(true, $id);
			$this->ajax->getResponse();
			return true;
		} else {
			$this->ajax->setResponse(false, 12);
			$this->ajax->getResponse();
			return false;
		}
	}

	public function markBuyer() {
		$contactId = intval($_POST['contactId']);
		$eventId = intval($_POST['eventId']);
		$productId = intval($_POST['productId']);
		$productPrice = $_POST['productPrice'];
		$howMuchPay = $_POST['howMuchPay'];
		$tagId = $_POST['tagId'];

		if (!$contactId) {
			$this->ajax->setResponse(false, 12);
			$this->ajax->getResponse();
			return false;
		}
		$this->log->add("Mark Buyer Called (Bootcamp) - ContactId:" . $contactId);

		$isoft = new Infusionsoft();

		if($productId){
			$order = $isoft->createPaidOrder('Event Sale', $contactId, $productId, $productPrice, $howMuchPay);

			if(!$order){
				$this->ajax->setResponse(false,12);
				$this->ajax->getResponse();
				return false;
			}
		}

		$resultTag = $isoft->applyTagById($contactId,$tagId);

		if($resultTag){
			$data = ['purchased' => 'Y'];
			$id = $this->db->updateRecord('registrants', $data, ['ContactId' => $contactId,'eventId'=>$eventId]);
		}else{
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
		if ($id) {
			$this->ajax->setResponse(true, $id);
			$this->ajax->getResponse();
			return true;
		} else {
			$this->ajax->setResponse(false, 12);
			$this->ajax->getResponse();
			return false;
		}
	}

	public function markAttendedLive(){
		$contactId = intval($_POST['contactId']);
		$tagId = $_POST['tagId'];
		$eventId = intval($_POST['eventId']);
		if(!$contactId || !$tagId || !$eventId){
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
		$this->log->add("Mark Attended Called (Live) - ContactId:".$contactId);

		$isoft = new Infusionsoft();
		$resultTag = $isoft->applyTagById($contactId,$tagId);

		if($resultTag){
			$data = ['attended' => 'Y'];
			$id = $this->db->updateRecord('registrants', $data, [ 'ContactId'=>$contactId,'eventId'=>$eventId ] );
		}else{
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
		if($id){
			$this->ajax->setResponse(true,$id);
			$this->ajax->getResponse();
			return true;
		}else{
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
	}

	public function createLiveContact(){
		$email = $_POST['email'];
		$eventId = intval($_POST['eventId']);
		$tags = $_POST['tags'];
		$this->log->add('New Contact Creation Called (Summit) - '.$email);
		if(!$email){
			$this->ajax->setResponse(false,11);
			$this->ajax->getResponse();
			return false;
		}

		$fields = [
			'Email'     => $email,
		];
		if(isset($_POST['firstname']) && $_POST['firstname']){
			$fields['FirstName'] = $_POST['firstname'];
		}
		if(isset($_POST['lastname']) && $_POST['lastname']){
			$fields['LastName'] = $_POST['lastname'];
		}
		if(isset($_POST['phone']) && $_POST['phone']){
			$fields['Phone1'] = $_POST['phone'];
		}

		$isoft = new Infusionsoft();
		$contactId = $isoft->findByEmail($email);

		if(!$contactId){
			$contactId = $isoft->createContact($fields);
		}else{
			$contactId = $isoft->updateContact($contactId,$fields);
		}
		$id = $isoft->applyTagById($contactId,$tags);
		if(!intval($id)){
			$this->ajax->setResponse(false,12);
		}else{
			$fields['contactId'] = $contactId;
			$fields['eventId'] = $eventId;
			$this->addContactToTableLive($fields);
			$this->ajax->setResponse(true,$id);
		}
		$this->ajax->getResponse();
		return true;
	}

	public function addContactToTableLive($data){
		$databaseValues = [
			'ContactId' => $data['contactId'],
			'attended' => 'Y',
			'eventId' => $data['eventId'],
			'VenueDate' => 'TBA'
		];
		if(isset($data['Email'])){
			$databaseValues ['Email'] = $data['Email'];
		}
		if(isset($data['FirstName'])){
			$databaseValues ['FirstName'] = $data['FirstName'];
		}
		if(isset($data['LastName'])){
			$databaseValues ['LastName'] = $data['LastName'];
		}
		if(isset($data['Phone1'])){
			$databaseValues ['Phone'] = $data['Phone1'];
		}
		$isExist = $this->db->hardSelect('SELECT * FROM `registrants` WHERE ContactId='.$databaseValues['ContactId'].' AND eventId='.$databaseValues['eventId']);
		if(empty($isExist)){
			$this->db->insert('registrants',$databaseValues);
		}else{
			$dataNew = ['attended' => 'Y'];
			$this->db->updateRecord('registrants', $dataNew, [ 'ContactId'=>$databaseValues['ContactId'] ] );
		}
		return true;
	}

	public function processSale(){
		$contactId = intval($_POST['contactId']);
		$tagId = intval($_POST['tagId']);
		$managerId = intval($_POST['managerId']);
		$this->log->add("Process Sale Called - UserID:".$contactId."; TagId:".$tagId."; ManagerId:".$managerId);
		if(!$contactId || !$tagId){
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
		$managerName = '';
		if($managerId){
			$manager = new model_manager();
			$managerData = $manager->getManager($managerId);
			$managerName = $managerData->firstName.' '.$managerData->lastName;
			$ownerId = intval($managerData->userID);
		}
		$isoft = new Infusionsoft();
		$resultTag = $isoft->applyTagById($contactId,$tagId);
		$resultFields = $isoft->updateContact($contactId,["_SalesManager"=>$managerName,"OwnerID"=>$ownerId]);
		if($resultTag && $resultFields){
			$this->ajax->setResponse(true,1);
			$this->ajax->getResponse();
			return false;
		}else{
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
	}

	public function updateEventInfo(){
		$doYouTradeFields = [
			1 => 'Full-time',
			2 => 'Part-time',
			3 => 'Not Yet'
		];

		$HowMuchDoYouNeedFields = [
			1 => '$0-$35000',
			2 => '$36-$55000',
			3 => '$56-$75000',
			4 => '$76-$100000',
			5 => '$100k+'
		];

		$contactId = intval($_POST['contactId']);

		$this->log->add("Update Info Called - ContactId:".$contactId);

		$CurrentOccupation = $_POST['_WhatisYourCurrentOccupation'];
		$DoYouTrade = isset($doYouTradeFields[intval($_POST['_DoYouTrade'])])? $doYouTradeFields[intval($_POST['_DoYouTrade'])] : false;
		$HowMuch = isset($HowMuchDoYouNeedFields[intval($_POST['_HowMuchDoYouNeed'])]) ? $HowMuchDoYouNeedFields[intval($_POST['_HowMuchDoYouNeed'])] : false;

		if(!$contactId || !$DoYouTrade || !$HowMuch){
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}
		$isoft = new Infusionsoft();
		$resultFields = $isoft->updateContact($contactId,["_DoYouTrade"=>$DoYouTrade,'_WhatisYourCurrentOccupation'=>$CurrentOccupation,'_HowMuchDoYouNeed'=>$HowMuch]);

		if(!$resultFields){
			$this->ajax->setResponse(false,12);
			$this->ajax->getResponse();
			return false;
		}else{
			$this->ajax->setResponse(true,$contactId);
			$this->ajax->getResponse();
			return false;
		}
	}

	public function regLog(){
		if($_SERVER['REQUEST_METHOD']!='POST'){
			return false;
		}
		$contact = [];
		$contact['firstName'] = $_POST['firstName'];
		$contact['lastName'] = $_POST['lastName'];
		$contact['email'] = $_POST['email'];
		$contact['phone'] = $_POST['phone'];
		$contact['id'] = $_POST['id'];
		$contact['source'] = $_POST['source'];

		$this->log->addReg($contact);
		return true;

	}
}