<?php
/*
 * core database
 */
Class Database{
    protected static $_instance = null;
    protected $database = array(
    'host' => DB_HOST,
    'database' => DATABASE,
    'user' => DB_USER,
    'password' => DB_PASSWORD,
    );
    protected $_columns = '';
    protected $_limit;
    protected $_where = '';
    protected $_join = '';
    protected $_table = '';
    protected $_groupBy = '';
    protected $_orderBy = '';
    protected $_distinct = false;
    protected $_query = '';
    protected $_params = '';
    protected $types = array(
        "integer"=>"i",
        "double"=>'d',
        "string"=>'s'
    );


    protected function __construct(){
        $this->db = new mysqli($this->database['host'],
            $this->database['user'],
            $this->database['password'],
            $this->database['database']);
        if ($this->db->connect_error) {
            die("Connection failed: " . $this->db->connect_error);
        }
    }

    protected function __clone(){}

    public static function getDatabase(){
        if (!self::$_instance instanceof self){
            self::$_instance = new self;
        }
        return self::$_instance;
    }

    public function getRecord(){

    }

    /*
     * @where : array
     * return - where string
     */
    public function formWhere($where,$delete = false){
        $typesStr = '';
        $whereStr = '';
        $params = array();
        $params[0]='';
        if ($where && is_array($where)){
            $whereStr = ' WHERE ';
            foreach( $where as $field => $value ){
                if ($delete){
                    $whereStr.= $field.'=? AND ';
                    if (isset($this->types[gettype($value)])){
                        $typesStr.=$this->types[gettype($value)];
                    }else{
                        $typesStr.='s';
                    }
                    $params[]=$value;
                }else{
                    $whereStr.= $field.'='.$value.' AND ';
                }
            }
            $whereStr = rtrim($whereStr,' AND ');
        }
        if($where && !is_array($where)){
            $whereStr = $where;
        }
        if ($delete){
            $params[0] = $typesStr;
            $result['WHERE'] = $whereStr;
            $result['params'] = $params;
            return $result;
        }else{
            return $whereStr;
        }
    }

    /*
     * Not actually delete record, just change deleted column to unix timestamp;
     */
    public function deleteRecord($table, $where){
        if (is_array($where)){
            return $this->updateRecord($table,array('deleted'=>time()),$where);
        }else{
            return false;
        }
    }

    /*
     * completely delete record from table
     */
    public function deleteFromDB($table,$where){
        $whereArr = $this->formWhere($where,true);
        $params = $whereArr['params'];
        $whereStr = $whereArr['WHERE'];
        $stmt = $this->db->prepare("DELETE FROM ".$table.$whereStr);
        call_user_func_array(array($stmt, 'bind_param'), $this->refValues($params));
        $stmt->execute();
        $stmt->close();
        return true;
    }

    public function hardDelete($query){
        $this->db->query($query);
        return true;
    }

    /*
     * start of making query, needed execute to complete query
     */
    public function getData($table, $columns){
        $this->_table = $table;
        $this->_columns = $columns;
        return $this;
    }

    /*
     * @where : string or array - filed=>value
     */
    public function where($where,$logic = 'AND'){

        if (!is_array($where)){
            $newWhere = array();
            if($logic == 'AND'){
                $where = explode('AND',$where);
            }else{
                $where = explode('OR',$where);
            }
            foreach($where as $line){
                $k = explode('=',$line);
                $k = $k[0];
                $v = explode('=',$line);
                $v = $v[1];
                $newWhere[$k] = $v;
            }
            $where=$newWhere;
        }
        if ($this->_where){
            $whereStr = $this->formWhere($where,true);
            $params = $whereStr['params'];
            $whereStr = $whereStr['WHERE'];
            $paramSring = array_shift($params);
            $this->_params = array_merge($this->_params,$params);
            $this->_params[0] = $this->_params[0].$paramSring;
            if($logic == 'AND'){
                $this->_where .= " AND ".ltrim($whereStr,' WHERE ');
            }else{
                $this->_where .= " OR ".ltrim($whereStr,' WHERE ');
            }
        }else{
            $this->_where = $this->formWhere($where,true);
            $this->_params = $this->_where['params'];
            $this->_where = $this->_where['WHERE'];
        }
        return $this;
    }

    public function limit($from,$to = false){
        if($to !== false){
        $this->_limit = " LIMIT {$from}, {$to}";
        }else{
            $this->_limit = " LIMIT {$from}";
        }
        return $this;
    }

    public function distinct($isTrue){
        if ($isTrue){
            $this->_distinct = " DISTINCT";
        }
        return $this;
    }

    public function join($table,$on,$type=''){
        if ($type){
            $type = $type." ";
        }
        $this->_join .= " ".$type."JOIN ".$table." ON ".$on;
        return $this;
    }

    public function groupBy($statement){
        $this->_groupBy = " GROUP BY ".$statement;
        return $this;
    }

    public function orderBy($statement){
        if($statement){
        $this->_orderBy = " ORDER BY ".$statement;
        }
        return $this;
    }

    /*
     * execute current query
     */
    public function execute(){
        $this->setQuery();
        $rows = $this->getQuery($this->_query);
        $this->nullVars();
        return $rows;
    }

    protected function nullVars(){
        $this->_query = "";
        $this->_table = "";
        $this->_join = "";
        $this->_where = "";
        $this->_groupBy = "";
        $this->_orderBy = "";
        $this->_limit = "";
    }

    /*
     * get array of objects of query
     */
    public function getQuery($query){
        $rows = array();
        if(!$this->_params){
            $result = $this->db->query($query);
            while ($obj = mysqli_fetch_object($result)) {
                $rows[] = $obj;
            }
        }else{
            //var_dump($this->_params); exit;
            $stmt = $this->db->prepare($query);
            call_user_func_array(array($stmt, 'bind_param'), $this->refValues($this->_params));
            $stmt->execute();
            $params=array();
            $c = new stdClass();
            $row = array();
            $meta = $stmt->result_metadata();
            while ($field = $meta->fetch_field())
            {
                $params[] = &$row[$field->name];
            }
            call_user_func_array(array($stmt, 'bind_result'), $params);
            while ($stmt->fetch()) {
                foreach($row as $key => $val)
                {
                    $c->$key = $val;
                }
                $rows[] = $c;
            }
            $stmt->close();
        }
        return $rows;
    }

    public function hardSelect($sql){
        $this->db->query($sql);
        $rows = array();
        $result = $this->db->query($sql);
        if($result){
            // Cycle through results
            while ($row = $result->fetch_object()){
                $rows[] = $row;
            }
    }
        return $rows;

    }

    /*
     * show current query
     */
    public function show(){
        $this->setQuery();
        echo $this->_query;
    }

    protected function setQuery(){
        $this->_query = "SELECT ".$this->_distinct." ".$this->_columns." FROM ".
            $this->_table.
            $this->_join.
            $this->_where.
            $this->_groupBy.
            $this->_orderBy.
            $this->_limit;
    }

    /*
     * @data - ['field'=>'value']
     */
    public function insert($table,$data){
        /*
         * variables initialize
         * @types : types of variable
         * @typesStr : string of types
         * @dataStF : fields that will be inserted
         * @dataStV : values that will be inserted
         */
        $params = array();
        $params[0]='';
        $typesStr = '';
        $dataStF = '(';
        $dataStV = '(';
        /*
         * forming fields, values and types string
         */
        foreach($data as $field => $value){
            $dataStF.=$field.', ';
            $dataStV.='?, ';
            if (isset($this->types[gettype($value)])){
                $typesStr.=$this->types[gettype($value)];
            }else{
                $typesStr.='s';
            }
            $params[]=$value;
        }
        $params[0] = $typesStr;
        $dataStF = rtrim($dataStF,', ').')';
        $dataStV = rtrim($dataStV,', ').')';
        $stmt = $this->db->prepare("INSERT INTO ".$table." ".$dataStF." VALUES ".$dataStV);
        call_user_func_array(array($stmt, 'bind_param'), $this->refValues($params));
        $stmt->execute();
        $id = $stmt->insert_id;
        $stmt->close();
        return $id;
    }

    protected function refValues($arr){
        if (strnatcmp(phpversion(),'5.3') >= 0) //Reference is required for PHP 5.3+
        {
            $refs = array();
            foreach($arr as $key => $value)
                $refs[$key] = &$arr[$key];
            return $refs;
        }
        return $arr;
    }


    /**
     * @table : string - table name
     * @data : array - field=>value
     * @where : array - field=>value
     */
    public function updateRecord($table, $data, $where = false){
        /*
         * variables initialize
         * @types : types of variable
         * @typesStr : string of types
         * @dataStF : fields that will be inserted
         * @dataStV : values that will be inserted
         */
        $params = array();
        $params[0]='';
        $typesStr = '';
        $valStr = '';

        /*
         * forming SET params and types string
         */
        foreach($data as $field => $value){
            $valStr .= $field.'= ?, ';
            if (isset($this->types[gettype($value)])){
                $typesStr.=$this->types[gettype($value)];
            }else{
                $typesStr.='s';
            }
            $params[]=$value;
        }
        $params[0] = $typesStr;
        $valStr = rtrim($valStr,', ');
        $whereStr = $this->formWhere($where);
        $stmt = $this->db->prepare("UPDATE ".$table." SET ".$valStr.$whereStr);
        call_user_func_array(array($stmt, 'bind_param'), $this->refValues($params));
        $stmt->execute();
        $stmt->close();
        return true;
    }

    public function __destruct(){
        $this->db->close();
    }

    public function simpleQuery($sql){
        $this->db->query($sql);
        return true;
    }

}