<?php
/*
 * core view
 */
Class View{
    public $header = false;
    public $footer = false;
    public $pageCurrent = 0;
    public $pageMax = 0;
    public $title = '';
    public $description = '';
    public $type = '';
    public $socImage = '';
    public $keywords = '';
    public $bodyclass = '';
    public $classString = '';
    public $headerScripts = [];
    public $footerScripts = [];

    /**
     * View constructor.
     */
    public function __construct(){
        header('Content-Type:text/html; charset=UTF-8');
        $headerFile = TEMPLATES.'header.php';
        $footerFile = TEMPLATES.'footer.php';
        if (file_exists($headerFile)){
            $this->header = $headerFile;
        }
        if (file_exists($footerFile)){
            $this->footer = $footerFile;
        }
        $this->navigation = new Navigation();
        return true;
    }

    public function renderTitle(){
        if ($this->title){
            echo '<title>'.$this->title.' - '.WEBSITE_TITLE.'</title>';
        }else{
            echo '<title>Rockwell Trading</title>';
        }
    }

    public function renderMeta(){

        if($this->description){
            echo '<meta property="og:description" content="'.$this->description.'" />';
            echo '<meta name="description" content="'.$this->description.'">';
        }
        echo '<meta property="og:url" content="'.CURRENT_LINK.'" />';

        if($this->title){
            echo '<meta property="og:title" content="'.$this->title.'" />';
            echo '<meta name="DC.title" content="'.$this->title.'">';
        }

        if($this->type){
            echo '<meta property="og:type" content="'.$this->type.'" />';
        }

        if($this->socImage){
            echo '<meta property="og:image" content="'.$this->socImage.'" />';
        }

        if($this->keywords){
            echo '<meta name="keywords" content="'.$this->keywords.'">';
        }
    }

    public function render($template,$data = false){
        include_once $this->header;
        require_once TEMPLATES.$template.'.php';
        include_once $this->footer;
    }

    public function renderBlank($template,$data = false){
        require_once TEMPLATES.$template.'.php';
     }

    public function renderStatic($template){
        require_once TEMPLATES.$template.'.php';
    }

    public static function set404(){
        header('HTTP/1.1 404 Not Found');
        header("Status: 404 Not Found");
        require_once PAGE404;
    }

    public static function redirectToHome(){
        header('location: /');
    }

    /**
     * @param string
     * @param array $optionsArray
     * @param string $selected
     * @param boolean $multiple
     * @param boolean $search
     * @param string $class
     * @return string
     */
    public static function generateSelect($fieldName, $optionsArray, $selected ,$multiple = false, $search=false ,$class=''){
        $searchString = $search?' data-live-search="true"' : '';
        $classString = $class?' class="'.$class.'"':'';
        $multipleString = $multiple?' multiple':'';
        $select = '<select name="'.$fieldName.'" id="'.$fieldName.'"'.$classString.$searchString.$multipleString.'>';
        $selectedFound = false;
        foreach($optionsArray as $value => $text){
            if ($value == $selected){
                $select .= '<option value="'.$value.'" selected="true">'.$text.'</option>';
                $selectedFound = true;
            }else{
                $select .= '<option value="'.$value.'">'.$text.'</option>';
            }
        }
        if(!$selectedFound){
            $selected = $selected > 0 ? $selected : 0;
            $text = $selected == 0 ? 'Not Selected' : '[Value Was Not Found]';
            $isSelected = $multiple ? '' : 'selected="true"';
            $select .= '<option '.$isSelected.' value="'.$selected.'">'.$text.'</option>';
        }
        $select.="</select>";
        return $select;
    }

    public function generateCheckbox($label,$checked,$name,$id=false,$class='form-check-input'){
    	$checkedStr = $checked?'checked=true':'';
		$checkbox = "<input class='".$class."' type='checkbox' id='".$id."' name='".$name."' ".$checkedStr.">";
		$label = "<label class='form-check-label' for='".$id."'>".$label."</label>";
		return $checkbox.$label;
    }

    /*
     * fieldName - field name and id
     * optionsArray - int value
     * selected - int value
     * id - leave empty if not specified
     * class - class of input
     */
    public static function generateRadio($fieldName,$value,$selected,$id='',$class=''){
        $idStr = '';
        $classStr = '';
        $selectedStr = '';
        if ($class){
            $classStr = ' class="'.$class.'"';
        }
        if ($id){
            $idStr = ' id="'.$id.'"';
        }
        if ($value == $selected){
            $selectedStr = ' checked';
        }
        $radio = '<input'.$idStr.' type="radio" name="'.$fieldName.'" value="'.$value.'" '.$classStr.$selectedStr.'/>';
        return $radio;
    }

    public function getTitle(){
        return $this->_title;
    }

    public function setTitle($title){
        $this->_title = $title;
    }

    public function setNavigation(){
        if($this->pageMax == 1){
            return false;
        }
        echo '<ul id="navigation" class="pagination">';
        if ($this->pageMax <= 3) {
            for ($i = 1; $i <= $this->pageMax; $i++) {
                if ($i == $this->pageCurrent) {
                    echo '<li class="nav-page active"><a href="?page=' . $i . '">' . $i . '</a></li>';
                } else {
                    echo '<li class="nav-page"><a href="?page=' . $i . '">' . $i . '</a></li>';
                }
            }
        } else {
            if ($this->pageCurrent > 3 && $this->pageCurrent - 3 - 1 != 1) {
                echo '<li class="nav-page"><a href="?page=1">1</a></li>';
                echo '<li class="nav-page"><a href="#">...</a></li>';
            } elseif ($this->pageCurrent - 4 == 1) {
                echo '<li class="nav-page"><a href="?page=1">1</a></li>';
            }
            for ($i = 1; $i <= $this->pageMax; $i++) {
                if (//$this->pageCurrent > 3 && $this->pageCurrent < $this->pageMax-3 &&
                    $this->pageCurrent - 3 != $i &&
                    $this->pageCurrent - 2 != $i &&
                    $this->pageCurrent - 1 != $i &&
                    $this->pageCurrent != $i &&
                    $this->pageCurrent + 1 != $i &&
                    $this->pageCurrent + 2 != $i &&
                    $this->pageCurrent + 3 != $i
                ) {
                    continue;
                }
                if ($i == $this->pageCurrent) {
                    echo '<li class="nav-page active"><a href="?page=' . $i . '">' . $i . '</a></li>';
                } else {
                    echo '<li class="nav-page"><a href="?page=' . $i . '">' . $i . '</a></li>';
                }
            }
            if ($this->pageCurrent < $this->pageMax - 3 && $this->pageCurrent + 3 + 1 != $this->pageMax) {
                echo '<li class="nav-page"><a href="#">...</a></li>';
                echo '<li class="nav-page"><a href="?page=' . $this->pageMax . '">' . $this->pageMax . '</a></li>';
            } elseif ($this->pageCurrent + 3 + 1 == $this->pageMax) {
                echo '<li class="nav-page"><a href="?page=' . $this->pageMax . '">' . $this->pageMax . '</a></li>';
            }
        }
        echo '</ul>';
        return true;
    }

    public function monthToAlpha($number){
        $month = array(
            1 => 'JAN',
            2 => 'FEB',
            3 => 'MAR',
            4 => 'APR',
            5 => 'MAY',
            6 => 'JUNE',
            7 => 'JULY',
            8 => 'AUG',
            9 => 'SEP',
            10 => 'OCT',
            11 => 'NOV',
            12 => 'DEC'
        );
        if (!isset($month[$number])){
            $number = 1;
        }
        return $month[$number];
    }

    /**
     * @param array $navigation
     */
    public function renderNavigation($navigation, $active = false){

    }

    public function setLayout($layoutName){
        if($layoutName == 'admin'){
            $this->header = TEMPLATES.'header-admin.php';;
            $this->footer = TEMPLATES.'footer-admin.php';
        }
    }

    /**
     * @param string $name: File Name
     * @return string
     */
    public function getImageURL($name){
        return BASE_URL.IMAGES_VIEW.$name;
    }

    public function setBodyClass($classString){
        if($this->classString == '') {
            $this->classString = $classString;
        }else{
            $this->classString .= ' '.$classString;
        }
        $this->bodyclass = 'class="'.$this->classString.'"';
    }

    public function addHeaderScripts($arrayOfPaths){
        if(!empty($arrayOfPaths)){
            foreach($arrayOfPaths as $path){
                $this->headerScripts[] = $path;
            }
        }
    }

    public function getHeaderScripts(){
        if(empty($this->headerScripts)){
            return '';
        }else{
            $output = '';
            foreach($this->headerScripts as $script){
                $output.= '<script src="'.$script.'"></script>';
            }
            return $output;
        }
    }

    public function addFooterScripts($arrayOfPaths){
        if(!empty($arrayOfPaths)){
            foreach($arrayOfPaths as $path){
                $this->footerScripts[] = $path;
            }
        }
    }

    public function getFooterScripts(){
        if(empty($this->footerScripts)){
            return '';
        }else{
            $output = '';
            foreach($this->footerScripts as $script){
                $output.= '<script src="'.$script.'"></script>';
            }
            return $output;
        }
    }

	/**
	 * @param $value - Y or NO string
	 * @param $checkOnly - Display only checked checkbox
	 * @return string - check mark
	 */
	public function checkMarkButton($value, $checkOnly = false){
		if($value == "Y"){
			return '<i class="fa fa-check-square button-green"></i>';
		}else{
			return $checkOnly ? '' : '<i class="fa fa-square-o"></i>';
		}
	}

	public function attendedBadge($value){
		if($value=='Y'){
			return '<span class="badge badge-success badge-attended">Attended</span>';
		}
		return '';
	}

	public function purchasedBadge($value){
		if($value=='Y'){
			return '<span class="badge badge-primary badge-purchased">Purchased</span>';
		}
		return '';
	}

}